import structure5.*;
/**
 * An iterator class for implementing an iterator over a Vector 
 * of numbers that returns the prefix sum of the numbers rather
 * than the numbers themselves.  That is, the sum of all values
 * up to and include the current.
 *
 * @version $Id: PrefixSumIterator.java 950 2009-10-07 02:09:52Z terescoj $
 * @author, Jim Teresco, 2009
 */
public class PrefixSumIterator extends AbstractIterator<Integer>
{
    /**
     * The associated vector
     */
    protected Vector<Integer> theVector;
    /**
     * The index of the current value.
     */
    protected int current;

    /**
       The prefix sum of the items encountered so far.
    */
    protected int prefixSum;

    /**
     * Construct a vector iterator to traverse vector v
     *
     * @post constructs an initialized iterator associated with v
     * 
     * @param v The underlying vector.
     */
    public PrefixSumIterator(Vector<Integer> v)
    {
        theVector = v;
        reset();
    }

    /**
     * Reset the vector iterator to the first value in the vector.
     *
     * @post the iterator is reset to the beginning of the traversal
     */
    public void reset()
    {
        current = 0;
	prefixSum = 0;
    }

    /**
     * Determine if some of the elements have yet to be considered.
     *
     * @post returns true if there is more structure to be traversed
     * 
     * @return True if more elements are to be considered.
     */
    public boolean hasNext()
    {
        return current < theVector.size();
    }

    /**
     * Fetch a reference to the current value.
     *
     * @pre traversal has more elements
     * @post returns the current value referenced by the iterator 
     * 
     * @return A reference to the current value being considered.
     */
    public Integer get()
    {
        return new Integer(prefixSum+theVector.get(current).intValue());
    }
    
    /**
     * Return current value, and increment iterator to next
     * value to be returned (the next even-numbered entry)
     *
     * @pre traversal has more elements
     * @post increments the iterated traversal
     * 
     * @return the prefix sum of the values up to the current
     */
    public Integer next()
    {
	prefixSum += theVector.get(current).intValue();
	current++;
        return new Integer(prefixSum);
    }
}

